package Acrobat::FDF;

use strict;
use Carp;
use FileHandle;
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter DynaLoader);
@EXPORT = qw(
FDFErcOK
FDFErcInternalError
FDFErcBadParameter
FDFErcFileSysErr
FDFErcBadFDF
FDFErcFieldNotFound
FDFErcNoValue
FDFErcEnumStopped
FDFErcCantInsertField
FDFErcNoOption
FDFErcNoFlags
FDFErcBadPDF
FDFErcBufTooShort
FDFErcNoAP
FDFErcIncompatibleFDF
FDFErcNoAppendSaves
FDFErcValueIsArray
FDFErcEmbeddedFDFs
FDFErcNoMoreFDFs
FDFErcInvalidPassword
FDFErcLast
FDFValue
FDFStatus
FDFFile
FDFID
FDFOpt
FDFFf
FDFSetFf
FDFClearFf
FDFFlags
FDFSetF
FDFClrF
FDFAP
FDFAS
FDFAction
FDFAA
FDFAPRef
FDFIF
FDFTargetFrame
FDFEncoding
FDFJavaScript
FDFAppendSaves
FDFNormalAP
FDFRolloverAP
FDFDownAP
FDFAlways
FDFTooSmall
FDFTooBig
FDFNever
FDFEnter
FDFExit
FDFDown
FDFUp
FDFFormat
FDFValidate
FDFKeystroke
FDFCalculate
FDFOnFocus
FDFOnBlur
true
false
);
$Acrobat::FDF::VERSION = '5.0';

sub AUTOLOAD {

    my $constname;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    my $val = constant($constname, @_ ? $_[0] : 0);
    if ($! != 0) {
	if ($! =~ /Invalid/) {
	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
	    goto &AutoLoader::AUTOLOAD;
	}
	else {
		croak "Your vendor has not defined FdfTk macro $constname";
	}
    }
    eval "sub $AUTOLOAD { $val }";
    goto &$AUTOLOAD;
}

sub FDFGetID {
        FDFGetID_internal(@_);  # makes use of @___TmpArray
        if($Acrobat::FDF::Erc == FDFErcOK()) { 
		return @Acrobat::FDF::___TmpArray; 
	}
        return undef;
}

sub FDFSetID {
        my $theFDF = shift;
        my $nElemNum = shift;

        @Acrobat::FDF::___TmpArray = @_;		# used by SetID_internal
        FDFSetID_internal($theFDF, $nElemNum);
}

sub FDFSetSubmitByNameAction {
	my $theFDF = shift;
	my $fieldName = shift;
	my $whichTrigger = shift;
	my $theURL = shift;
	my $theFlags = shift;

	@Acrobat::FDF::___TmpArray = @_;
	FDFSetSubmitByNameAction_internal($theFDF, $fieldName, $whichTrigger, $theURL, $theFlags);
}

sub FDFSetResetByNameAction {
        my $theFDF = shift;
        my $fieldName = shift;
        my $whichTrigger = shift;
        my $theFlags = shift;

        @Acrobat::FDF::___TmpArray = @_;
        FDFSetResetByNameAction_internal($theFDF, $fieldName, $whichTrigger, $theFlags);
}

sub FDFSetValues {
        my $theFDF = shift;
        my $fieldName = shift;

        @Acrobat::FDF::___TmpArray = @_;
        FDFSetValues_internal($theFDF, $fieldName );
}

# complain on failure
sub onFail {
	if($Acrobat::FDF::Erc != FDFErcOK()) { croak(shift); }
}

# OO wrapper

# note that there are no functions corresponding to Create, Open, OpenFromBuf
# or Close; the constructors and destructor are the only way of creating and
# destroying FDF objects.

# member variables:
# $theFDF => the FDFDoc IV
# $Erc => the last error code

# constructors
sub new {
	my $class = shift;
	my $fname = shift;
	my $howmany = shift;
	my $this = {};
	
	if(!($Acrobat::FDF::fdfDocCount++)) {
		FDFInitialize();
		if($Acrobat::FDF::Erc != FDFErcOK()) { return undef; }
	}
	
	if(defined $fname) { $this->{theFDF} = FDFOpen($fname, $howmany); }
	else { $this->{theFDF} = FDFCreate(); }
	
	if(($this->{Erc} = $Acrobat::FDF::Erc) != FDFErcOK()) { return undef; }
	
	bless $this;
	return $this;
}

sub newFromBuf {
	my $class;
	my $buf;
	my $this;
	my $tmpfile;
	my $tmpname;
	my $time = time();

	if($#_ == 1) { $class = shift }
	$buf = shift;
	if(($tmpname = $ENV{TMP}) eq '') { $tmpname = '/tmp'; }
	$tmpname .= "/FdfTknFB.$$.$time.fdf";

	if(open($tmpfile, ">$tmpname") == undef) {
		$this->{Erc} = Acrobat::FDF::FDFFileSysErr();
		return undef;
	}
	binmode $tmpfile;
	
	if(!$tmpfile->print($buf)) {
		$this->{Erc} = Acrobat::FDF::FDFFileSysErr();
		$tmpfile->close;
		if(!(unlink $tmpfile))
			{ warn("Couldn't delete temp file!"); }		
		return undef;
	}
	$tmpfile->close;
	undef $tmpfile;
	$this = new Acrobat::FDF($tmpname,0);
	$this->{toUnlink} = $tmpname;
	return $this;
}
	 
# destructor
sub DESTROY {
	my $this = shift;

	FDFClose($this->{theFDF});
	onFail('Unable to close FDFDoc');

	if(defined($this->{toUnlink}))
	{
		if(!(unlink $this->{toUnlink}))
			{ warn("Couldn't delete temp file!"); }
	}
	
	if(!(--$Acrobat::FDF::fdfDocCount)) { FDFFinalize(); }
	onFail('Unable to finalize toolkit');
}

# static methods (in addition to the ones from the XS)

# kind of redundant, but keeps the convention
sub GetVersion {
	return FDFGetVersion();
}

# regular methods

# "private"
sub setErr {
	my $this = shift;

	$this->{Erc} = $Acrobat::FDF::Erc;
}

# "public"

sub LastError {
	my $this = shift;

	return $this->{Erc};
}

sub Save {
	my $this = shift;

	FDFSave($this->{theFDF},@_);
	$this->setErr();
}

sub NextFieldName {
	my $this = shift;
	my $fname;

	$fname = FDFNextFieldName($this->{theFDF},@_);
	$this->setErr();
	return $fname;
}

sub GetValue {
	my $this = shift;
	my $value;

	$value = FDFGetValue($this->{theFDF},@_);
	$this->setErr();
	return $value;
}

sub NewFromEmbedded {
    my $this = shift;
    my $this2 = {};

	if(!($Acrobat::FDF::fdfDocCount++)) {
		FDFInitialize();
		if($Acrobat::FDF::Erc != FDFErcOK()) { return undef; }
	}
	
    $this2->{theFDF} = FDFOpenFromEmbedded($this->{theFDF},@_);
    if(($this2->{Erc} = $Acrobat::FDF::Erc) != FDFErcOK()) { return undef; }
    
    bless $this2;
    return $this2;
}

sub EmbedAndClose {
    my $this = shift;
    my $embedded = shift;
    
    FDFEmbedAndClose($this->{theFDF}, $embedded->{theFDF}, @_);
    if(($this->{Erc} = $Acrobat::FDF::Erc) == FDFErcOK()) {
	 $embedded->{theFDF} = 0;
	 undef $embedded;
    }

    return $this;
}

sub GetNthValue {
	my $this = shift;
	my $value;

	$value = FDFGetNthValue($this->{theFDF},@_);
	$this->setErr();
	return $value;
}

sub SetValue {
	my $this = shift;

	FDFSetValue($this->{theFDF},@_);
	$this->setErr();
}

sub GetStatus {
	my $this = shift;
	my $status;

	$status = FDFGetStatus($this->{theFDF});
	$this->setErr();
	return $status;
}

sub GetFDFVersion {
	my $this = shift;
	my $ver;

	$ver = FDFGetFDFVersion($this->{theFDF});
	$this->setErr();
	return $ver;
}

sub SetStatus {
	my $this = shift;
	
	FDFSetStatus($this->{theFDF},@_);
	$this->setErr();
}

sub SetTargetFrame {
	my $this = shift;
	
	FDFSetTargetFrame($this->{theFDF},@_);
	$this->setErr();
}

sub ExtractAttachment {
	my $this = shift;
	my @ret;
	
	@ret = FDFExtractAttachment($this->{theFDF},@_);
	$this->setErr();
	
	return @ret;
}

sub ExtractAppendSaves {
	my $this = shift;
	
	FDFExtractAppendSaves($this->{theFDF},@_);
	$this->setErr();
}

sub SetFDFVersion {
	my $this = shift;
	
	FDFSetFDFVersion($this->{theFDF},@_);
	$this->setErr();
}

sub AddDocJavaScript {
	my $this = shift;
	
	FDFAddDocJavaScript($this->{theFDF},@_);
	$this->setErr();
}

sub SetOnImportJavaScript {
	my $this = shift;
	
	FDFSetOnImportJavaScript($this->{theFDF},@_);
	$this->setErr();
}
sub GetFile {
	my $this = shift;
	my $fname;

	$fname = FDFGetFile($this->{theFDF});
	$this->setErr();
	return $fname;
}

sub SetFile {
	my $this = shift;

	FDFSetFile($this->{theFDF},@_);
	$this->setErr();
}

sub GetID {
	my $this = shift;
	my @id;

	@id = FDFGetID($this->{theFDF},@_);
	$this->setErr();
	return @id;
}

sub SetID {
	my $this = shift;
	
	FDFSetID($this->{theFDF},@_);
	$this->setErr();
}

sub GetOpt {
	my $this = shift;
	my @opt;

	@opt = FDFGetOpt($this->{theFDF},@_);
	$this->setErr();
	return @opt;
}

sub SetOpt {
	my $this = shift;

	FDFSetOpt($this->{theFDF},@_);
	$this->setErr();
}

sub GetFlags {
	my $this = shift;
	my $flags;

	$flags = FDFGetFlags($this->{theFDF},@_);
	$this->setErr();
	return $flags;
}

sub SetFlags {
	my $this = shift;
	
	FDFSetFlags($this->{theFDF},@_);
	$this->setErr();
}

sub SetAPRef {
	my $this = shift;

	FDFSetAPRef($this->{theFDF},@_);
	$this->setErr();
}

sub SetIF {
	my $this = shift;

	FDFSetIF($this->{theFDF},@_);
	$this->setErr();
}

sub SetFileEx {
	my $this = shift;

	FDFSetFileEx($this->{theFDF},@_);
	$this->setErr();
}

sub GetAP {
	my $this = shift;

	FDFGetAP($this->{theFDF},@_);
	$this->setErr();
}

sub SetAP {
	my $this = shift;

	FDFSetAP($this->{theFDF},@_);
	$this->setErr();
}

sub SetAS {
	my $this = shift;

	FDFSetAS($this->{theFDF},@_);
	$this->setErr();
}

sub AddTemplate {
	my $this = shift;

	FDFAddTemplate($this->{theFDF},@_);
	$this->setErr();
}

sub SetJavaScriptAction {
	my $this = shift;

	FDFSetJavaScriptAction($this->{theFDF},@_);
	$this->setErr();
}

sub SetSubmitByNameAction {
	my $this = shift;

	FDFSetSubmitByNameAction($this->{theFDF}, @_);
	$this->setErr();
}

sub SetSubmitFormAction {
	my $this = shift;

	FDFSetSubmitFormAction($this->{theFDF},@_);
	$this->setErr();
}

sub SetResetByNameAction {
	my $this = shift;

	FDFSetResetByNameAction($this->{theFDF},@_);
	$this->setErr();
}

sub SetValues {
	my $this = shift;

	FDFSetValues($this->{theFDF},@_);
	$this->setErr();
}

sub SetResetFormAction {
	my $this = shift;

	FDFSetResetFormAction($this->{theFDF},@_);
	$this->setErr();
}

sub SetImportDataAction {
	my $this = shift;

	FDFSetImportDataAction($this->{theFDF},@_);
	$this->setErr();
}

sub SetGoToAction {
	my $this = shift;

	FDFSetGoToAction($this->{theFDF},@_);
	$this->setErr();
}

sub SetGoToRAction {
	my $this = shift;

	FDFSetGoToRAction($this->{theFDF},@_);
	$this->setErr();
}

sub SetURIAction {
	my $this = shift;
	
	FDFSetURIAction($this->{theFDF},@_);
	$this->setErr();
}

sub SetNamedAction {
	my $this = shift;

	FDFSetNamedAction($this->{theFDF},@_);
	$this->setErr();
}

sub SetHideAction {
	my $this = shift;

	FDFSetHideAction($this->{theFDF},@_);
	$this->setErr();
}

sub SetStateAction {
	my $this = shift;

	FDFSetStateAction($this->{theFDF},@_);
	$this->setErr();
}

if(0) # removed
{
sub SetNOPAction {
	my $this = shift;

	FDFSetNOPAction($this->{theFDF},@_);
	$this->setErr();
}
}

sub RemoveItem {
	my $this = shift;

	FDFRemoveItem($this->{theFDF},@_);
	$this->setErr();
}

sub GetEncoding {
	my $this = shift;
	my $encoding;

	$encoding = FDFGetEncoding($this->{theFDF});
	$this->setErr();
	return $encoding;
}

sub SetEncoding {
	my $this = shift;

	FDFSetEncoding($this->{theFDF},@_);
	$this->setErr();
}

sub SaveToBuf {
	my $this = shift;
	my $tmpname;
	my $tmpfile = new FileHandle;
	my $buf;
	my $oldDelim;
	my $time = time();
	
	if(($tmpname = $ENV{TMP}) eq '') { $tmpname = '/tmp'; }
	$tmpname .=  "/FdfTkSTB.$$.$time.fdf";

	$this->Save($tmpname);
	if($this->LastError() != Acrobat::FDF::FDFErcOK()) { return undef; }
	
	if(open($tmpfile, "<$tmpname") == undef) {
		$this->{Erc} = Acrobat::FDF::FDFErcFileSysErr();
		return undef;
	}
	binmode($tmpfile);
	
	$oldDelim = $/;
	undef $/;
	if(($buf = join(//,<$tmpfile>)) eq '') {
		$this->{Erc} = Acrobat::FDF::FDFErcFileSysErr();
		return undef;
	}
	$/ = $oldDelim;
	undef $tmpfile;
	if(!(unlink $tmpname)) { warn("Couldn't delete temp file!"); }
	return $buf;
}

package Acrobat::FDF::Hash;

# tied hash object
sub TIEHASH {
	my $class = shift;
	my $fdfObj = shift;
	my $this = {};

	bless $this;
	if(!defined($fdfObj->{theFDF})) { return undef; }	# isa?
	$this->{fdfObj} = $fdfObj;
	return $this;
}

sub FETCH {
	my $this = shift;

	return $this->{fdfObj}->GetValue(@_);
}

sub STORE {
	my $this = shift;

	$this->{fdfObj}->SetValue(@_,0);
}

sub DELETE {
	my $this = shift;

	$this->{fdfObj}->RemoveItem(@_,FDFValue());
}

sub CLEAR {
	my $this = shift;
	my $fdfObj = $this->{fdfObj};
	my $fname;

	while(($fname = $fdfObj->NextFieldName('')) ne '') {
		$fdfObj->RemoveItem($fname,FDFValue());
	}
}

sub EXISTS {
	my $this = shift;

	$this->{fdfObj}->GetValue(@_);
	return $this->{fdfObj}->LastError() == FDFErcOK();
}

sub FIRSTKEY {
	my $this = shift;

	return $this->{fdfObj}->NextFieldName('');
}

sub NEXTKEY {
	my $this = shift;


	return $this->{fdfObj}->NextFieldName(@_);
}

bootstrap Acrobat::FDF $Acrobat::FDF::VERSION;

1;
__END__

