/*
* AppendUpdate.c

ADOBE SYSTEMS INCORPORATED
Copyright [1999] - [2001] Adobe Systems Incorporated
All Rights Reserved

NOTICE:  Adobe permits you to use, modify, and distribute 
this file in accordance with the terms of the Adobe license 
agreement accompanying it.  If you have received this file 
from a source other than Adobe, then your use, modification, 
or distribution of it requires the prior written permission of Adobe.


  Requires write access for the saved update section. Saved at:
		c:\inetpub\wwwroot\PDFs\tmp\

*/

#define _WIN32_WINNT 0x0400 

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef SOLARIS
#include <thread.h>
#endif

#ifdef WIN32
#include <io.h>
#include <fcntl.h>
#include "Windows.h"
#endif

#include "FdfTk.h"


/*
  These definitions are needed only on Unix, for multithreading
*/
#ifdef SOLARIS
#define INITIALIZE_FDF_THREADSAFE { \
   static mutex_t mutexObj; \
   mutex_init( &mutexObj, NULL, NULL ); \
   FDFRegisterThreadsafeCallbacks( \
      (ThreadsafeCallback)mutex_lock, \
      (ThreadsafeCallback)mutex_unlock, \
      (ThreadsafeCallback)mutex_destroy, &mutexObj ); }
#endif

#ifndef INITIALIZE_FDF_THREADSAFE
#define INITIALIZE_FDF_THREADSAFE
#endif


/*
  The error handling strategy we use in this example is to return to the client
  an indication of which call into the FDF API failed, and what the error code
  was. This is probably a good approach during the debugging phase, but not for
  production systems!
*/
static void HandleError(FDFErc theErc, const char* funcName, const char* parameter, FDFDoc inFDF)
{
    if (theErc != FDFErcOK) {
	if (inFDF) FDFClose(inFDF);
	printf("Content-type: text/plain\n\n");
	printf("Error %d in call to %s. Parameter: %s\n", theErc, funcName, parameter);

#ifdef UNIX_ENV /* Do not need to do this on Win32 */
	FDFFinalize();
#endif

	exit(1);
    }
}


static void CopyFileToStdout(const char* cFilename)
{
    ASBool bKeepReading = true;
    FILE* f = fopen(cFilename, "rb");

    while (bKeepReading) {
	char cBuf[8192];
	size_t nRead = fread(cBuf, sizeof(char), sizeof(cBuf), f);

	if (nRead)
	    fwrite(cBuf, sizeof(char), nRead, stdout);

	bKeepReading = (nRead == sizeof(cBuf));
    }

    fclose(f);
}


void main(int argc, char *argv[], char *envp[])
{
    const char* cTempFileSaveAppends = "c:\\inetpub\\wwwroot\\PDFs\\tmp\\AppendUpdate.tmp";
    const char* cPdfFileName = "c:\\inetpub\\wwwroot\\PDFs\\AppendUpdate.pdf";
    FDFDoc inFDF = NULL;
    FDFErc theErc;
	ASInt32 howMany = atoi(getenv("CONTENT_LENGTH"));

	

#ifdef UNIX_ENV
    /*
      On Unix only, need to define the platform-dependent functions used for
      threading.
      This particular example does not use threading, but we still wanted
      to show how the FDFRegisterThreadsafeCallbacks() API is used.
    */
    INITIALIZE_FDF_THREADSAFE;
    /*
      This example assumes a cgi-bin environment, where each invocation of
      the program spawns a new process. In a multi-threaded environment,
      FDFInitialize() should only be called once, by the first thread in the
      application. Once again, this is Unix only. On Win32 the initialization
      and multithreading stuff is taken care of automatically by the FdfTk.dll
    */
    FDFInitialize();
#endif

    theErc = FDFOpen("-", howMany, &inFDF);
    HandleError(theErc, "FDFOpen", getenv("CONTENT_LENGTH"), inFDF);
    theErc = FDFExtractAppendSaves(inFDF, cTempFileSaveAppends);
    HandleError(theErc, "FDFExtractAppendSaves", "", inFDF);
    FDFClose(inFDF);

#ifdef WIN32
    _setmode(_fileno(stdout), _O_BINARY);
#endif

	  printf("Content-type: application/pdf\n\n");
    fflush(stdout); /* Emit the HTTP header now! */

    CopyFileToStdout(cPdfFileName);
    CopyFileToStdout(cTempFileSaveAppends);
    remove(cTempFileSaveAppends);

#ifdef UNIX_ENV
    /*
      This example assumes a cgi-bin environment, where each invocation of
      the program spawns a new process. In a multi-threaded environment,
      FDFFinalize() should only be called once, by the last thread in the
      application. Once again, this is Unix only. On Win32 the initialization
      and multithreading stuff is taken care of automatically by the FdfTk.dll
    */
    FDFFinalize();
#endif

    exit(0);
}
