#ADOBE SYSTEMS INCORPORATED
#Copyright [1999] - [2001] Adobe Systems Incorporated
#All Rights Reserved

#NOTICE:  Adobe permits you to use, modify, and distribute 
#this file in accordance with the terms of the Adobe license 
#agreement accompanying it.  If you have received this file 
#from a source other than Adobe, then your use, modification, 
#or distribution of it requires the prior written permission of Adobe.

# This sample is composed of the following files:
# employee_query.pl, employee_update.pl, and blank.pl are the three source files, they live in a folder
#	with execute permissions, e.g. C:\InetPub\scripts
# employee_db.txt: this is our "database", and consists of one record per line, with comma separated
#	fields (see @schema below). This file is assumed to be in the same folder as the perl files.
# wkshp_employee.pdf: this is the Acrobat Form, and it can be placed, for example, in C:\InetPub\wwwroot
# Blank.pdf: this is a blank pdf page, used by blank.pl. See below.
# photos: this is a folder where the employee pictures will be located, and its path is assumed to be
#	C:\InetPub\wwwroot\photos

# When the employee first opens the Form in the browser (e.g. by going to the URL
#	<http://yourserver.com/wkshp_employee.pdf>), only the ssn field is visible. The employee types
#	in his SSN and clicks on QUERY. This sends an FDF with the value of the ssn field to
#	employee_query.pl, which tries to locate the record in employee_db.txt. If found, the data in the
#	record is used to populate an FDF which is sent back to the client. If not, an FDF is sent back
#	which informs the user (via a text field called "status") of this fact. In either case, all fields
#	previously hidden in the Form are made available, and the user can insert/update his record by
#	submitting to employee_update.pl. Also, the QUERY button gets hidden and the UPDATE button is shown
#	in its place. The user may include a photograph (in PDF) when submitting the data to the server,
#	and this photo is available next time a query is initiated. The photo is saved as a PDF file in
#	the photos folder on the server, under the last name of the employee. The CLEAR button on the Form
#	not only triggers a ResetForm action, but also triggers a submit to blank.pl. This script returns
#	an FDF which clears the appearance of the "photo" button on the Form. CLEAR also hides and shows
#	various fields to restore the Form to its initial state. There is a third button on the Form with
#	the caption SAVE. This is used to illustrate how the employee could save the partially-filled Form
#	to a file (only if using Exchange, not the free Reader), and then work offline with it, and submit
#	it to the server at a later time by reopening it in the browser. Finally, at the upper right-hand
#	corner of the Form there is a PFN ("Personal Field Names") button that uses the Import Form Data
#	action to import the user's myprof.fdf, which saves the user from having to retype frequently
#	requested form information. 



use Acrobat::FDF;

@schema = ('ssn', 'name.first', 'name.initial', 'name.last', 'home.address.line1',
		   'home.address.line2', 'home.address.line3', 'home.address.city', 'home.address.state',
		   'home.address.zip', 'home.telephone.voice.areacode', 'home.telephone.voice.number',
		   'birthdate.month', 'birthdate.day', 'birthdate.year', 'gender', 'photo');

$outFdf = new Acrobat::FDF; # Create an FDF for the response

if (!defined $outFdf) {
	print "Content-type: text/plain\n\n";
	print "Could not create response FDF\n";
	exit;
}

binmode STDIN; # FDFs contain binary data. On Windows, this statement is important, not so on Unix.
read STDIN, $buf, $ENV{'CONTENT_LENGTH'};
$inFdf = Acrobat::FDF::newFromBuf($buf); # Read the FDF POSTed from the client

if (!defined $inFdf) {
	print "Content-type: text/plain\n\n";
	print "POSTed data is malformed\n";
	exit;
}

$ssn = $inFdf->GetValue("ssn"); # The received FDF should include the ssn field

if (!($lastErr = $inFdf->LastError())) { # Use LastError for error checking
	@attributes = findRecord($ssn);

	if ($#attributes > -1) { # We found the record!
        foreach $attr (@attributes) {
			$key = shift @schema;

			# gender is a radio button, therefore pass "true" as the last parameter to SetValue. The
			# same is true for checkboxes. For other field types, pass "false"
			$outFdf->SetValue($key, $attr, $key eq 'gender');

			if (($lastErr = $outFdf->LastError())) {
				last;
			}
        }

		if (!$lastErr) {

			# We have (at least) 2 choices for how to send a status message back:
			# Either use the SetStatus call, which causes an alert to pop up at the client:
			# $outFdf->SetStatus('Make any changes as necessary, and then click on "UPDATE"');
			# or use a text field to display a message. Since the "status" field in wkshp_employee.pdf
			# has no border and no background color, and it is read-only, it is invisible until some
			# text is written to it via this FDF.
			$outFdf->SetValue('status', 'Make any changes as necessary, and then click on "UPDATE"', 0);
		}

		if (!$lastErr) {
			# Send the photograph back to the client via the FDF. The photo is under the last name
			# of the employee.
			$outFdf->SetAP('photo', FDFNormalAP, 0,
						   'C:\\InetPub\\wwwroot\\PDFs\\photos\\' . $outFdf->GetValue('name.last') . '.pdf', 1);
		}
	}
	else {
		$outFdf->SetValue('status', 'Record not found. Please enter your data and click on "UPDATE"', 0);
	}
}

if (!$lastErr) {
	$outFdf->SetFlags('overlay', FDFSetF, 2); # Hide the overlay field

	if (!($lastErr = $outFdf->LastError())) {
		$outFdf->SetFlags('Submit.Query', FDFSetF, 2); # Hide the Submit.Query field
	}

	if (!($lastErr = $outFdf->LastError())) {
		$outFdf->SetFlags('Reset', FDFClrF, 2); # Show the Reset field
	}

	if (!($lastErr = $outFdf->LastError())) {
		$outFdf->SetFlags('Save', FDFClrF, 2); # Show the Save field
	}
}

if ($lastErr) {
	print "Content-type: text/plain\n\n";
	print Win32::FormatMessage $lastErr; 
}
else {
	print "Content-type: application/vnd.fdf\n\n";

	 binmode STDOUT;
	 print $outFdf->SaveToBuf();
}

sub findRecord {
        my $ssn = shift;

		open EMP_DB, 'c:\InetPub\scripts\employee_db.txt' or return ();

		while ($record = <EMP_DB>) {
			chop($record);
			if ($record =~ /^$ssn/) {
				return split /,/, $record;
			}
		}

		close EMP_DB;
        return ();
}
