/************************************************************************

  FramesAndFDF.cpp
 
ADOBE SYSTEMS INCORPORATED
Copyright [1999] - [2001] Adobe Systems Incorporated
All Rights Reserved

NOTICE:  Adobe permits you to use, modify, and distribute 
this file in accordance with the terms of the Adobe license 
agreement accompanying it.  If you have received this file 
from a source other than Adobe, then your use, modification, 
or distribution of it requires the prior written permission of Adobe.



**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*	
	The line below includes the standard FDF Toolkit Header file. 
*/

#include "Fdftk.h"

/*
	Main application
*/

void main()

{
	FDFErc	retCode;
	FDFDoc	outputFDF = NULL;


/*	
	Create a new FDF. Parameter is the pointer to FDFDoc every call uses. 	
*/
	retCode = FDFCreate (&outputFDF);	
		
/* 
	Now, take the pointer to the FDFDoc just created and add a new field data. This 
	sample application set new values in the FDF file created using the FDFSetValue. m

	You could generate your own values by parsing a database or getting input 
	from the user then call the methods below to build the FDF data. In this sample
	we'll generate them using simple strings.
	
	FDFSetValue uses the following parameters: 

		theFDF		= Valid FDFDoc returned from FDFCreate or FDFOpen.
		fieldName	= Null-terminated string representing the fully qualified
					name of the field (e.g. "employee.name.last").
		newValue	= Null-terminated string to use as new value.
		isName		= If true, newValue is converted to a PDF Name before
					making it the value of the field. If false, newValue is set 
					to a PDF String.
*/

	retCode = FDFSetValue (outputFDF, "status", "Thanks George!", false); 

/* 
	Set the file for the outgoing fdf. This sets the name of the PDF file
	that will be used by this application to populate the FDF. When the user
	clicks on the opening PDF pages "Get Data" button, it will run this app 
	and populate the same PDF. 
*/
	retCode = FDFSetFile(outputFDF, "http://localhost/PDFs/simple.pdf");
	
/* 
	Set the target frame for the PDF. This is similar to setting the target
	in an anchor in HTML.
*/
	retCode = FDFSetTargetFrame(outputFDF, "Right");
/* 
	Next we'll do three things: 

	1)  Set everything up to emit the correct HTTP header for the MIME type. In 
		the case with FDF, the MIME needs to be set to "application/vnd.fdf".
	2)	Emit the HTTP header
	3)	Write the FDF data to stdout
*/
    printf("Content-type: application/vnd.fdf\n\n");
    fflush(stdout); 
    FDFSave (outputFDF,  "-");	

/*	
	You don't have to make this next call. It just shows that after the data is
	flushed to standard out you may want to save it locally. you can then open 
	the FDF by clicking on it or opening it with Acrobat. It's alot easier than
	saving the whole PDF with the data. 

	We want a record of the transaction, so save the FDF data to a specific place 
	on the hard drive.  
*/
//	 FDFSave (outputFDF, "C:\\InetPub\\wwwroot\\Pdf\\generated-data.fdf"); 


/*	
	Close the FDF File used to generate the output.
*/
	retCode = FDFClose (outputFDF);		

}