/*
ADOBE SYSTEMS INCORPORATED
Copyright [1999] - [2001] Adobe Systems Incorporated
All Rights Reserved

NOTICE:  Adobe permits you to use, modify, and distribute 
this file in accordance with the terms of the Adobe license 
agreement accompanying it.  If you have received this file 
from a source other than Adobe, then your use, modification, 
or distribution of it requires the prior written permission of Adobe.

**
**	parseFDF - A sample Windows NT Server application to parse FDF data from "stdin". 
**
**	There are three files required for building your application with C/C++, FDFTK.H, 
**	FDFTK.LIB, and FDFTK.DLL. I addition to the standard header files, Include the FDF 
**	Toolkit's header file as well. Copy FDFTK.H and FDFTK.LIB from the distribution into 
**	the same directory as your project. Include  FDFTK.LIB in the link options. Copy 
**	FDFTK.DLL into the same directory as the sample application.
*/

#include <stdio.h>
#include <stdlib.h>
#include "fdftk.h"


void main()

	{

	/*
	**	Definitions
	*/
	
	FDFDoc FdfInput;
	FDFErc errorCode;
	char cNameBuffer [50];
	char cAddrBuffer [50];
	char cComboBuffer [50];

	ASInt32 howMany = atoi(getenv("CONTENT_LENGTH"));

	/*
	**	FDFOpen:
	**	Open the FDF file from "stdin". FDFOpen parameters:
	**
	**	filename		- Pathname to the FDF file, if you want to read from "stdin" use "-"
	**	homMany			- If reading from "stdin", this is the number of bytes to read
	**	pinFDF			- Pointer to an FDFDoc. This will be used by the rest of the API.
	*/

	errorCode = FDFOpen ("-", howMany, &FdfInput);
	
	/* 
	** FDFGetValue:
	**
	**	This gets the value of the /V key (the value of the specified field). Use the data 
	**	returned in the buffer for your application (e.g. the address or name of the customer).
	**	
	*/

	errorCode = FDFGetValue (FdfInput, "Name", cNameBuffer, sizeof(cNameBuffer), &howMany);
	 
	errorCode = FDFGetValue (FdfInput, "Address", cAddrBuffer, sizeof(cAddrBuffer), &howMany);
	
	errorCode = FDFGetValue (FdfInput, "ListBox", cComboBuffer, sizeof(cComboBuffer), &howMany);

	/*
	** Presumably after we parsed this data, we would propulate a database or generate another
	** FDF file with some sort of response ...
	*/

		// Your code would go here

	/*
	**	This next line of code is important if you are returning FDF from the server. If you are
	**  parsing FDF coming in from "stdin" and returning FDF, you must emit the correct 
	**  MIME type to "stdout" or you'll get a CGI error simular to this:
	**
	**		"The specified CGI application misbehaved by not returning  
	**		 a complete set of HTTP headers."
	**  
	**  At this point you would generate FDF for the return and then do this:
	**
	**
	**		printf ("Content-type: application/vnd.fdf\n\n");
	**		fflush (stdout);
	**
	** 
	**  For this example we will only send back acknowledgement that the code worked
	*/ 
	
		printf ("Content-type: text/plain\n\n");
		printf ("Results are:\n");
		printf ("Customer Name: %s\n", cNameBuffer);
		printf ("Customer Address: %s\n", cAddrBuffer);
		printf ("ListBox selection export value: %s\n", cComboBuffer);
		

	/*
	** FDFClose:
	**
	**	Use FDFClose to close any open FDFDocs and free resources used by the library. 
	**	You must call FDFClose. FDFClose parameters:
	**	
	**		inFDF		- The FDFDoc returned by FDFOpen or FDFCreate
	*/

	errorCode = FDFClose(FdfInput);


}

