/*
	PingForm.cpp
	
ADOBE SYSTEMS INCORPORATED
Copyright [1999] - [2001] Adobe Systems Incorporated
All Rights Reserved

NOTICE:  Adobe permits you to use, modify, and distribute 
this file in accordance with the terms of the Adobe license 
agreement accompanying it.  If you have received this file 
from a source other than Adobe, then your use, modification, 
or distribution of it requires the prior written permission of Adobe.
*/

#include <stdlib.h>
#include <malloc.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include "FdfTk.h"

/*
  The error handling strategy we use in this example is to return to the client
  an indication of which call into the FDF API failed, and what the error code
  was. This is probably a good approach during the debugging phase, but not for
  production systems!
*/
static void HandleError(FDFErc theErc, const char* funcName, const char* parameter,
						FDFDoc outFDF)
{
    if (theErc != FDFErcOK) {
		if (outFDF) FDFClose(outFDF);
		printf("Content-type: text/plain\n\n");
		printf("Error %d in call to %s. Parameter: %s\n", theErc, funcName, parameter);

		exit(1);
    }
}


void main(int argc, char *argv[], char *envp[])
{
    FDFDoc outFDF = NULL;
    FDFErc theErc;
    ASInt32 howMany = atoi(getenv("CONTENT_LENGTH"));
	char* cBuf = (char*) malloc(howMany + 1);

    /* Create a new FDF to send the response to the client */
    theErc = FDFCreate(&outFDF);
    HandleError(theErc, "FDFCreate", "none", outFDF);

    /*
      Read into a buffer the FDF data submitted from the client. This example assumes
	  cgi-bin, where the POSTed data is available at stdin, and its length is in
      environment variable CONTENT_LENGTH.
    */
	_setmode(_fileno(stdin), _O_BINARY);
	fread(cBuf, sizeof(char), howMany, stdin);
	cBuf[howMany] = '\0';

	theErc = FDFSetValue(outFDF, "Ping Result", cBuf, false);
	free(cBuf);
	HandleError(theErc, "FDFSetValue", "Ping Result", outFDF);

    /*
      Don't forget to emit the correct HTTP header for the MIME type.
      By the way, remember when setting the URL of a SubmitForm action, to end
      it in #FDF. For example, the URL used in pingform.pdf to submit to this
	  cgi-bin program is: localhost/Scripts/pingform.cgi#FDF
    */
    printf("Content-type: application/vnd.fdf\n\n");
    fflush(stdout); /* Emit the HTTP header now! */

    FDFSave(outFDF, "-"); /* Now send the FDF to stdout */
    FDFClose(outFDF);

    exit(0);
}
